PROGRAM SkinTut3;

{**********************************************************}
{                                                          }
{   Skins program tutorial 3                               }
{                                                          }
{   Copyright (c) 2002 by Leon de Boer                     }
{   ldeboer@attglobal.net  - primary e-mail address        }
{   ldeboer@iprimus.com.au - backup e-mail address         }
{                                                          }
{****************[ THIS CODE IS FREEWARE ]*****************}
{                                                          }
{     This sourcecode is released for the purpose to       }
{   promote the pascal language on all platforms. You may  }
{   redistribute it and/or modify with the following       }
{   DISCLAIMER.                                            }
{                                                          }
{     This SOURCE CODE is distributed "AS IS" WITHOUT      }
{   WARRANTIES AS TO PERFORMANCE OF MERCHANTABILITY OR     }
{   ANY OTHER WARRANTIES WHETHER EXPRESSED OR IMPLIED.     }
{                                                          }
{*****************[ SUPPORTED PLATFORMS ]******************}
{                                                          }
{     FreePascal 1.0.6+     Windows 32 Bit                 }
{                                                          }
{*****************[ REVISION HISTORY ]*********************}
{  Version  Date        Fix                                }
{  -------  ---------   ---------------------------------  }
{  1.00     17 Jun 02    First release                     }
{**********************************************************}
{    Now on any skins program buttons are generally not of }
{  the usual press button form that was on tutorial 2 but  }
{  are generally much fancier "hover buttons" or sometimes }
{  referred to as "mouse over buttons". In the basic form  }
{  these buttons are not part of the skin but as we will   }
{  show in this example they can also be changed just like }
{  the main form to be part of the skin.                   }
{    A hover button can be made by capturing the mouse or  }
{  by using a timer and polling the status. Both of these  }
{  techniques have merit, in this case I chose the capture }
{  method only because I feel it is easier to follow. The  }
{  status of the button is held in a data byte "hovermode" }
{  which I have attached as a property to the window. This }
{  byte could have been a global variable it was simply my }
{  choice to attach it as a property because when we go to }
{  a framework we will have many buttons and there is no   }
{  confusion as to the "hovermode" byte for each window.   }
{  What we lose in simplicity in this simple example we    }
{  will easily make up for on the more complex examples.   }
{    Our hover button has three states each of which will  }
{  have a bitmap. They are normal, hover and press modes.  }
{  As our button is going to have a shape we also need a   }
{  shape mask which will give the button shape. In this    }
{  example we are going to use the normal bitmap also as   }
{  the shape mask because in our case all three bitmaps    }
{  will have the same shape. A more complex scheme would   }
{  have a mask bitmap which may include background parts   }
{  on each button but this is a complexity we will ignore  }
{  for now.                                                }
{    Our button is going to have an command id of 100 so   }
{  I have called our bitmaps N100.BMP, H100.BMP, P100.BMP  }
{  which represent normal, hover and pressed states. Again }
{  later we will combine these into 1 file but for now so  }
{  it is easier to follow they will be left as 3 files.    }
{                                                          }
{**********************************************************}

{$APPTYPE GUI}                                        { FULL GUI MODE }
USES Windows;

CONST
   szClassName  = 'SKINTUT3';                         { The app class name }
   szHoverClass = 'HOVERBUTTON';                      { Hover button class name }


CONST
   NORMALBMP = 'NORMALBITMAP';                        { Normal bmp property name }
   HOVERBMP  = 'HOVERBITMAP';                         { Hover bmp property name }
   PRESSBMP  = 'PRESSBITMAP';                         { Press bmp property name }

CONST
   BUTTONRGN = 'HOVERBUTTONREGION';                   { Hover button region }

CONST
   HOVERMODE = 'HOVERMODEBYTE';                       { Hover mode property name }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  LoadBmpFromFile -> Load bitmap from file          Updated -> 9May02 LdB  }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  This is a pascal port of a C function workaround for the LoadImage API   }
{  call by Microsoft (Q158898). The function reads a BMP file and returns   }
{  a hanle to the loaded bitmap and the palette for the bitmap.             }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
FUNCTION LoadBmpFromFile (FileName: String; VAR Pal: HPalette): HBitmap;
VAR I: Integer; szFileName: String; Bmp, OldBmp: HBitMap; Bm: TBitMap;
    MemDc: hDc; RGB: Array [0..255] Of TRGBQuad; LogPal: PLogPalette;
BEGIN
   szFileName := FileName + #0;                       { Make asciiz name }
   Bmp := LoadImage(HInstance, @szFilename[1],
     IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR OR
     LR_CREATEDIBSECTION OR LR_LOADFROMFILE);         { Load bitmap }
   IF (Bmp <> 0) THEN BEGIN
     GetObject(Bmp, SizeOf(Bm), @Bm);                 { Get bitmap data }
     IF ((Bm.bmBitsPixel * Bm.bmPlanes) <= 8) THEN
     BEGIN                                            { 256 colours or less }
       MemDc := CreateCompatibleDc(0);                { Create memory DC }
       OldBmp := SelectObject(MemDc, Bmp);            { Select this bitmap }
       GetDIBColorTable(MemDc, 0, 256, RGB);          { Get color table }
       GetMem(LogPal, SizeOf(TLogPalette) +
         (256*SizeOf(TPaletteEntry)));                { Allocate a memory }
       LogPal^.palVersion := $0300;                   { Set palette version }
       LogPal^.palNumEntries := 256;                  { Set entry count }
       FOR I := 0 TO 255 DO
         WITH LogPal^ DO BEGIN
           palPalEntry[I].peRed := RGB[I].rgbRed;     { Transfer red colour }
           palPalEntry[I].peGreen := RGB[I].rgbGreen; { Transfer green colour }
           palPalEntry[I].peBlue := RGB[I].rgbBlue;   { Transfer blue colour }
           palPalEntry[I].peFlags := 0;               { Clear flag entry }
         END;
       Pal := CreatePalette(LogPal^);                 { Create a palette }
       FreeMem(LogPal, SizeOf(TLogPalette) +
         (256*SizeOf(TPaletteEntry)));                { Release memory }
       SelectObject(MemDc, OldBmp);                   { Restore old bitmap }
       DeleteObject(MemDc);                           { Release context }
     END ELSE BEGIN
       MemDc := GetDc(0);                             { Get ref context }
       Pal := CreateHalftonePalette(MemDc);           { Create palette }
       ReleaseDc(0, MemDc);                           { Release context }
     END;
   END;
   LoadBmpFromFile := Bmp;                            { Return the bitmap }
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  BitmapToRegion -> Create region from bitmap       Updated -> 8May02 LdB  }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  This is a pascal port of a C function by Jean-Edouard Lachand-Robert     }
{  (http://www.geocities.com/Paris/LeftBank/1160/resume.htm), June 1998.    }
{  It takes a bitmap and creates a region based upon the pixels in the      }
{  bitmap matching a color that corresponds to Transcolor with a            }
{  Color Tolerance level of Tolerance.                                      }
{  It does this by scanning each line in the BITMAP structure for           }
{  pixels that match TransColor then adds them to the region with a         }
{  call to CombineRgn.                                                      }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
FUNCTION BitmapToRegion(BmpName: String; TransColor: TColorRef; Tolerance: Byte): HRGN;
CONST ALLOC_UNIT = 100;
VAR B, LR, LG, LB, HR, HG, HB: Byte; X, X0, Y: LongInt; MaxRects: DWord;
    pbits32: Pointer; MemDC, TempDc: HDc; BitmapInfo: TBitmapInfo; Pal: HPalette;
    InBmp, hbm32: HBitmap; Bm, bm32: TBITMAP; hData: HGLOBAL; pData: PRgnData;
    p32: pByte; p: pLongInt; pr: PRect; h: HRGN; HoldObj, HoldObj1: HGDIObj;
    OldPal: HPalette; Rslt: HRGN;
BEGIN
  Rslt := 0;                                          { Preset fail result }
  InBmp := LoadBmpFromFile(BmpName, Pal);             { Load main bitmap }
  IF (InBmp <> 0) THEN BEGIN                          { Bitmap loaded }
    MemDC := CreateCompatibleDC(0);                   { Create memory context }
    IF (MemDC <> 0) THEN BEGIN                        { Memory context valid }
      GetObject(InBmp, SizeOf(Bm), @Bm);              { Get bitmap data }
      WITH BitmapInfo.bmiHeader DO BEGIN              { Make bitmap header }
        biSize          := SizeOf(TBitmapInfoHeader); { Set record size }
        biWidth         := Bm.bmWidth;                { Width of bitmap }
        biHeight        := Bm.bmHeight;               { Height of bitmap }
        biPlanes        := 1;                         { 1 plane }
        biBitCount      := 32;                        { 32 bit DIB }
        biCompression   := BI_RGB;                    { uncompressed format }
        biSizeImage     := 0;                         { Default size }
        biXPelsPerMeter := 0;                         { Default x res }
        biYPelsPerMeter := 0;                         { Default y res }
        biClrUsed       := 0;                         { No unused colours }
        biClrImportant  := 0;                         { No important colours }
      END;
      hbm32 := CreateDIBSection(MemDC, BitmapInfo,
        DIB_RGB_COLORS, pbits32, 0, 0);               { Create a DIB section }
      IF (hbm32 <> 0) THEN BEGIN
        HoldObj := SelectObject(MemDC, hbm32);        { Select DIB section }
        TempDC := CreateCompatibleDC(0);              { Create memory DC }
        HoldObj1 := SelectObject(TempDC, InBmp);      { Select bitmap }
        OldPal := SelectPalette(TempDc, Pal, True);   { Select palette }
        RealizePalette(TempDc);                       { Realize the palette }
        BitBlt(MemDC, 0, 0, Bm.bmWidth, Bm.bmHeight,
          TempDc, 0, 0, SRCCOPY);                     { Transfer bitmap }
        SelectPalette(TempDc, OldPal, False);         { Restore old palette }
        SelectObject(TempDc, HoldObj1);               { Select original object }
        DeleteObject(TempDc);                         { Delete memory context }
        GetObject(hbm32, SizeOf(bm32), @bm32);        { Fetch bitmap data }
        WHILE (bm32.bmWidthBytes MOD 4) > 0 DO
          Inc(bm32.bmWidthBytes);                     { Make divisable by 4 }
        MaxRects := ALLOC_UNIT;                       { Set maximum size }
        hData := GlobalAlloc(GMEM_MOVEABLE, SizeOf(TRgnDataHeader) +
          (SizeOf(TRect) * MaxRects));                { Allocate memory }
        pData := GlobalLock(hData);                   { Lock the memory }
        pData^.rdh.dwSize := SizeOf(TRgnDataHeader);  { Set memory size }
        pData^.rdh.iType := RDH_RECTANGLES;           { Rectangular area }
        pData^.rdh.nCount := 0;                       { Zero count }
        pData^.rdh.nRgnSize := 0;                     { Zero region size }
        SetRect(pData^.rdh.rcBound, MaxInt, MaxInt,
          0, 0);                                      { Make rect maximum }
        LR := GetRValue(TransColor);                  { Lowest red }
        LG := GetGValue(TransColor);                  { Lowest green }
        LB := GetBValue(TransColor);                  { Lowest blue }
        If (LR + Tolerance > $FF) Then HR := $FF      { Check not above max }
          ELSE HR := LR + Tolerance;                  { New high red value }
        If (LG + Tolerance > $FF) Then HG := $FF      { Check not above max }
          ELSE HG := LG + Tolerance;                  { New high green value }
        If (LB + Tolerance > $FF) Then HB := $FF      { Check not above max }
          ELSE HB := LB + Tolerance;                  { New high blue value }
        p32 := bm32.bmBits;                           { Pointer to bmp data }
        Inc(PChar(p32), LongInt(bm32.bmHeight - 1) *
          LongInt(bm32.bmWidthBytes));                { Move to bottom line }
        FOR Y := 0 to Bm.bmHeight-1 DO BEGIN          { Each line of bitmap }
          X := -1;                                    { Start left }
          WHILE (X+1 < Bm.bmWidth) DO BEGIN           { For each pixel }
            Inc(X);                                   { Next x position }
            X0 := X;                                  { Hold left x value }
            p := PLongInt(p32);                       { Hold pointer }
            Inc(PChar(p), X * SizeOf(LongInt));       { Set p start value }
            WHILE (X < Bm.bmWidth) DO BEGIN           { For each pixel }
              B := GetBValue(p^);                     { Get blue value }
              IF (B >= LR) AND (B <= HR) THEN BEGIN   { Check for match }
                B := GetGValue(p^);                   { Get green value }
                IF (B >= LG) AND (B <= HG) THEN BEGIN
                  B := GetRValue(p^);                 { Get red value }
                  IF (B >= LB) AND (B <= HB) THEN     { Pixel is "transparent" }
                    Break;                            { Just exit }
                END;
              END;
              Inc(PChar(P), SizeOf(LongInt));         { Advance pointer }
              Inc(X);                                 { Next x position }
            END;
            IF (X > X0) THEN BEGIN                    { New region to add }
              IF (pData^.rdh.nCount >= MaxRects)
              THEN BEGIN                              { Area is to big }
                GlobalUnlock(hData);                  { Unlock memory }
                Inc(MaxRects, ALLOC_UNIT);            { Adjust size }
                hData := GlobalReAlloc(hData, SizeOf(TRgnDataHeader) +
                  SizeOf(TRect) * MaxRects, GMEM_MOVEABLE);
                pData := GlobalLock(hData);           { Relock memory }
              END;
              pr := @pData^.Buffer[pData^.rdh.nCount * SizeOf(TRect)];
              SetRect(Pr, X0, Y, X, Y+1);            { Set rectangle coords }
              IF (X0 < pData^.rdh.rcBound.Left) THEN
                pData^.rdh.rcBound.Left := X0;        { Check left x value }
              IF (Y < pData^.rdh.rcBound.Top) THEN
                pData^.rdh.rcBound.Top := Y;          { Check top y value }
              IF (X > pData^.rdh.rcBound.Right) THEN
                pData^.rdh.rcBound.Left := X;         { Check right x value }
              IF (Y+1 > pData^.rdh.rcBound.Bottom) THEN
                pData^.rdh.rcBound.Bottom := Y+1;     { Check bottom y value }
              Inc(pData^.rdh.nCount);                 { Add 1 region }
              IF (pData^.rdh.nCount = 2000)           { Reach maximum region count }
              THEN BEGIN                              { Region count at max }
                h := ExtCreateRegion(NIL, SizeOf(TRgnDataHeader) +
                  (SizeOf(TRect)*MaxRects), pData^);  { Create a region }
                IF (Rslt <> 0) THEN BEGIN             { Already have region }
                  CombineRgn(Rslt, Rslt, h, RGN_OR);
                  DeleteObject(h);                    { Delete old region }
                END ELSE Rslt := h;                   { Hold region }
                pData^.rdh.nCount := 0;               { Rezero region count }
                SetRect(pData^.rdh.rcBound, MaxInt,
                  MaxInt, 0, 0);                      { Reset rectangle }
              END;
            END;
          END;
          Dec(PChar(p32), bm32.bmWidthBytes);         { Move one line up }
        END;
        h := ExtCreateRegion(NIL, SizeOf(TRgnDataHeader) +
          (SizeOf(TRect) * MaxRects), pData^);        { Create a region }
        IF (Rslt <> 0) THEN BEGIN                     { Region already exists }
          CombineRgn(Rslt, Rslt, h, RGN_OR);          { Combine regions }
          DeleteObject(h);                            { Delete old region }
        END ELSE Rslt := h;                           { Return this region }
        GlobalFree(hData);                            { Clean up data }
        SelectObject(MemDc, HoldObj);                 { Select original object }
        DeleteObject(hbm32);                          { Delete dib section }
      END;
      DeleteDC(MemDC);                                { Delete device context }
    END;
    DeleteObject(Pal);                                { Destroy palette }
    DeleteObject(InBmp);                              { Delete bitmap object }
  END;
  BitmapToRegion := Rslt;                             { Return the region }
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  HoverButtonHandler -> Bitmap button handler       Updated -> 17Jun02 LdB }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
FUNCTION HoverButtonHandler (Wnd: hWnd; iMsg, wParam, lParam: LongInt): LongInt; STDCALL;
VAR ButtonId, HoverState: Integer; BmpName: String; R, R1: TRect; MemDc: hDc;
    Bmp: HBitmap; Pal: HPalette; Bm: TBitmap; Ps: TPaintStruct; Rgn: HRgn;
BEGIN
   HoverButtonHandler := 0;                           { Preset return }
   CASE iMsg OF
     WM_CREATE: BEGIN                                 { WM_CREATE MESSAGE }
       ButtonId := GetDlgCtrlId(Wnd);                 { Many buttons later! }
       Str(ButtonId, BmpName);                        { Button id to string }
       BmpName := 'N' + BmpName + '.BMP';             { "N100.BMP" }
       Bmp := LoadBmpFromFile (BmpName, Pal);         { Load bitmap }
       GetObject(Bmp, SizeOf(Bm), @Bm);               { Get bitmap data }
       GetWindowRect(Wnd, R);                         { Get our rect }
       GetWindowRect(GetParent(Wnd), R1);             { Get parent rect }
       MoveWindow(Wnd, R.Left-R1.Left, R.Top-R1.Top,
         Bm.bmWidth, Bm.bmHeight, False);             { Adjust button size }
       MemDC := CreateCompatibleDC(0);                { Memory DC }
       SelectObject(MemDc, Bmp);                      { Select bitmap }
       SelectPalette(MemDc, Pal, False);              { Select palette }
       RealizePalette(MemDc);                         { Realize the palette }
       SetProp(Wnd, NORMALBMP, MemDc);                { Save DC as property }
       Str(ButtonId, BmpName);                        { Button id to string }
       BmpName := 'H' + BmpName + '.BMP';             { "H100.BMP" }
       Bmp := LoadBmpFromFile (BmpName, Pal);         { Load bitmap }
       MemDC := CreateCompatibleDC(0);                { Memory DC }
       SelectObject(MemDc, Bmp);                      { Select bitmap }
       SelectPalette(MemDc, Pal, False);              { Select palette }
       RealizePalette(MemDc);                         { Realize the palette }
       SetProp(Wnd, HOVERBMP, MemDc);                 { Save DC as property }
       Str(ButtonId, BmpName);                        { Button id to string }
       BmpName := 'P' + BmpName + '.BMP';             { "P100.BMP" }
       Bmp := LoadBmpFromFile (BmpName, Pal);         { Load bitmap }
       MemDC := CreateCompatibleDC(0);                { Memory DC }
       SelectObject(MemDc, Bmp);                      { Select bitmap }
       SelectPalette(MemDc, Pal, False);              { Select palette }
       RealizePalette(MemDc);                         { Realize the palette }
       SetProp(Wnd, PRESSBMP, MemDc);                 { Save DC as property }
       HoverState := 1;                               { Button is normal }
       SetProp(Wnd, HOVERMODE, HoverState);           { Save hover mode property }
       Str(ButtonId, BmpName);                        { Button id to string }
       BmpName := 'N' + BmpName + '.BMP';             { "N100.BMP" }
       Rgn := BitmapToRegion(BmpName, RGB($FF, $0, $FF), $1);
       IF (Rgn <> 0) THEN BEGIN                       { Region was created }
         SetWindowRgn(Wnd, Rgn , False);              { Set widow to region }
         Rgn := BitmapToRegion(BmpName, RGB($FF, 0, $FF), $1);
         SetProp(Wnd, BUTTONRGN, Rgn);                { Save region as property }
       END;
     END;
     WM_DESTROY: BEGIN                                { WM_DESTROY MESSAGE }
       RemoveProp(Wnd, HOVERMODE);                    { Remove hover mode property }
       Rgn := GetProp(Wnd, BUTTONRGN);                { Region from property }
       IF (Rgn <> 0) THEN DeleteObject(Rgn);          { If valid dispose }
       RemoveProp(Wnd, BUTTONRGN);                    { Remove region property }
       MemDc := GetProp(Wnd, PRESSBMP);               { MemDc from property }
       IF (MemDc <> 0) THEN DeleteObject(MemDc);      { If valid dispose }
       RemoveProp(Wnd, PRESSBMP);                     { Remove press bmp property }
       MemDc := GetProp(Wnd, HOVERBMP);               { MemDc from property }
       IF (MemDc <> 0) THEN DeleteObject(MemDc);      { If valid dispose }
       RemoveProp(Wnd, HOVERBMP);                     { Remove hover bmp property }
       MemDc := GetProp(Wnd, NORMALBMP);              { MemDc from property }
       IF (MemDc <> 0) THEN DeleteObject(MemDc);      { If valid dispose }
       RemoveProp(Wnd, NORMALBMP);                    { Remove normal bmp property }
     END;
     WM_LBUTTONDOWN: BEGIN                            { WM_LBUTTONDOWN MESSAGE}
       SetCapture(Wnd);                               { Capture mouse input }
       HoverState := 3;                               { Button is depressed }
       SetProp(Wnd, HOVERMODE, HoverState);           { Save hover property }
       InvalidateRect(Wnd, Nil, False);               { Redraw button }
     END;
     WM_LBUTTONUP: BEGIN                              { WM_LBUTTONUP MESSAGE}
       HoverState := GetProp(Wnd, HOVERMODE);         { Get hover property }
       IF (HoverState = 3) THEN PostMessage(GetParent(Wnd),
         WM_COMMAND, GetDlgCtrlId(Wnd), 0);           { If down send message }
       HoverState := 1;                               { Button is released }
       SetProp(Wnd, HOVERMODE, HoverState);           { Save hover property }
       ReleaseCapture;                                { Release mouse capture }
       InvalidateRect(Wnd, Nil, False);               { Redraw button }
     END;
     WM_MOUSEMOVE: BEGIN                              { WM_MOUSEMOVE MESSAGE }
       Rgn := GetProp(Wnd, BUTTONRGN);                { Region from property }
       HoverState := GetProp(Wnd, HOVERMODE);         { Get hover property }
       IF (HoverState <> 0) AND (Rgn <> 0) THEN BEGIN { Valid entries }
         IF PtInRegion(Rgn,  LOWORD(lParam),
           HIWORD(lParam)) THEN BEGIN                 { Point in region }
           IF (HoverState = 1) THEN BEGIN             { As yet unhovered }
             IF (wparam AND (mk_lButton OR mk_rButton) = 0)
               THEN HoverState := 2
               Else HoverState := 3;                  { Select hover mode }
             SetProp(Wnd, HOVERMODE, HoverState);     { Set hover property }
             SetCapture(Wnd);                         { Capture mouse input }
             InvalidateRect(Wnd, Nil, False);         { Redraw button }
           END;
         END ELSE BEGIN                               { Point out of region }
           HoverState := 1;                           { Button is normal }
           SetProp(Wnd, HOVERMODE, HoverState);       { Save hover property }
           ReleaseCapture;                            { Release mouse capture }
           InvalidateRect(Wnd, Nil, False);           { Redraw button }
         END;
       END;
     END;
     WM_PAINT: BEGIN                                  { WM_PAINT MESSAGE }
       GetWindowRect(Wnd, R);                         { Get area }
       HoverState := GetProp(Wnd, HOVERMODE);         { Get hover property }
       CASE HoverState OF
         1: MemDc := GetProp(Wnd, NORMALBMP);         { DC from property }
         2: MemDc := GetProp(Wnd, HOVERBMP);          { DC from property }
         3: MemDc := GetProp(Wnd, PRESSBMP);          { DC from property }
         ELSE MemDc := 0;                             { Zero context }
       END;
       IF (MemDc <> 0) THEN BEGIN                     { MemDC is valid }
         BeginPaint(Wnd, Ps);                         { Begin paint }
         BitBlt(Ps.hDc, 0, 0, R.Right-R.Left,
           R.Bottom-R.Top, MemDc, 0, 0, SRCCOPY);     { BitBlt image }
         EndPaint(Wnd, Ps);                           { End paint }
       END;
     END;
     ELSE HoverButtonHandler := DefWindowProc(Wnd,
       iMsg, wParam, lParam);                         { Call default handler }
   END;
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  SkinHandler -> Main window handler                Updated -> 8May02 LdB  }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  Pretty basic windows handler. On creation it loads the bitmap image to   }
{  a device context which it uses in the paint call. On destruction it then }
{  frees this bitmap. The paint message simply paints the bitmap.           }
{  The hittest is returned so it always drags and the close button command  }
{  is accepted and closes the app.                                          }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
FUNCTION SkinHandler(Wnd: HWND; iMsg, wParam, lParam: LongInt): LongInt; STDCALL;
VAR R: TRect; MemDc: HDc; BitMap: HBitmap; Bm: TBitmap; Ps: TPaintStruct;
    Pal: HPalette;
BEGIN
   SkinHandler := 0;                                  { Preset zero return }
   Case iMsg of
     WM_CREATE: BEGIN                                 { WM_CREATE MESSAGE }
       MemDC := CreateCompatibleDC(0);                { Create memory DC }
       BitMap := LoadBmpFromFile('SKINTUT3.BMP', Pal);{ Load bitmap }
       DeleteObject(Pal);                             { Destroy created palette }
       GetObject(Bitmap, SizeOf(Bm), @Bm);            { Get bitmap data }
       SelectObject(MemDC, Bitmap);                   { Select bitmap }
       SetProp(Wnd, 'DCWITHBITMAP', MemDc);           { Set memdc to property }
       MoveWindow(Wnd, 0, 0, Bm.BmWidth, Bm.BmHeight, False);
     END;
     WM_DESTROY: BEGIN                                { WM_DESTROY MESSAGE }
       MemDc := GetProp(Wnd, 'DCWITHBITMAP');         { Retreive property }
       If (MemDc <> 0) THEN DeleteObject(MemDc);      { Dispose context }
       PostQuitMessage(0);                            { Post quit message }
     END;
     WM_PAINT: BEGIN                                  { WM_PAINT MESSAGE }
       GetWindowRect(Wnd, R);
       MemDc := GetProp(Wnd, 'DCWITHBITMAP');         { Retreive property }
       BeginPaint(Wnd, Ps);                           { Get paint structure }
       BitBlt(Ps.hDc, 0, 0, R.Right-R.Left,
         R.Bottom-R.Top, MemDC, 0, 0, SRCCopy);       { Copy bitmap screen }
       EndPaint(Wnd, Ps);                             { End of painting }
     END;
     WM_NCHITTEST: BEGIN                              { WM_NCHITTEST MESSAGE }
       SkinHandler := htCaption;                      { Return caption hit }
     END;
     WM_COMMAND: Begin                                { WM_COMMAND MESSAGE }
       CASE LoWord(wParam) Of                         { Low word wparam is cmd }
         100: PostQuitMessage(0);                     { Close button id }
         ELSE Skinhandler := DefWindowProc(Wnd, iMsg,
           wParam, lParam);                           { Call default handler }
       END;
     END;
     ELSE Skinhandler := DefWindowProc(Wnd, iMsg,
       wParam, lParam);                               { Call default handler }
   END;
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{  Main program code start                           Updated -> 8May02 LdB  }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
VAR Li: LongInt; Wnd: HWND; Msg: TMsg; Wndclass: TWndClassEx;
BEGIN
   If NOT GetClassInfoEx(0, szHoverClass, WndClass)   { Check not registered }
   THEN BEGIN
     WITH Wndclass DO BEGIN                           { With record }
       cbSize        := Sizeof(WndClass);             { Record size }
       style         := 0;                            { No special style }
       lpfnWndProc   := @HoverButtonHandler;          { Handler to use }
       cbClsExtra    := 0;                            { No extra colors }
       cbWndExtra    := 0;                            { No extra data }
       hInstance     := System.hInstance;             { Set instance }
       hIcon         := LoadIcon(0, IDI_APPLICATION); { Normal icon }
       hCursor       := LoadCursor(0, IDC_ARROW) ;    { Normal cursor }
       hbrBackground := GetStockObject(Null_Brush);   { No backbround }
       lpszMenuName  := Nil;                          { No menu }
       lpszClassName := szHoverClass;                 { Class name }
       hIconSm       := LoadIcon(0, IDI_APPLICATION); { Normal small icon }
     End;
     RegisterClassEx(Wndclass);                       { Register the class }
   END;
   If NOT GetClassInfoEx(0, szClassName, WndClass)    { Check not registered }
   THEN BEGIN
     WITH Wndclass DO BEGIN                           { With record }
       cbSize        := Sizeof(Wndclass);             { Record size }
       style         := 0;                            { No special style }
       lpfnWndProc   := @SkinHandler;                 { Handler to use }
       cbClsExtra    := 0;                            { No extra colors }
       cbWndExtra    := 0;                            { No extra data }
       hInstance     := System.hInstance;             { Set instance }
       hIcon         := LoadIcon(0, IDI_APPLICATION); { Normal icon }
       hCursor       := LoadCursor(0, IDC_ARROW) ;    { Normal cursor }
       hbrBackground := GetStockObject(Null_Brush);   { No backbround }
       lpszMenuName  := Nil;                          { No menu }
       lpszClassName := szClassName;                  { Class name }
       hIconSm       := LoadIcon(0, IDI_APPLICATION); { Normal small icon }
     End;
     RegisterClassEx(Wndclass);                       { Register the class }
   END;
   Wnd := CreateWindowEx(0, szClassName, 'SKIN DEMO',
     0, CW_USEDEFAULT,
     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
     0, 0, hInstance, Nil) ;                          { Create skin window }

   {* we must take caption off the window *}
   Li := GetWindowLong(Wnd, gwl_style);               { Get current style }
   Li := Li AND NOT WS_CAPTION;                       { Turn caption off }
   Li := Li AND NOT WS_ThickFrame;                    { Turn frame off }
   SetWindowLong(Wnd, gwl_style, Li);                 { Set the window style }


   {* make the window the desired shape by cutting out selected colour(s) *}
   {Ref1 -> Change the RGB values to cut another colour out }
   SetWindowRgn(Wnd,
     BitmapToRegion('SKINTUT3.BMP',
     RGB($FF, 0, $FF), $1),                           { Colour to cut }
     False);                                          { Set rgn from bmp to window }

   ShowWindow(Wnd, sw_ShowNormal);                    { Show window }
   UpdateWindow(Wnd);                                 { Update window }

   CreateWindowEx(0, szHoverClass, 'EXIT',
     WS_VISIBLE OR WS_CHILD,
     150, 215, 0, 0,
     Wnd, 100, hInstance, Nil);                       { Create exit button }

   WHILE (GetMessage(Msg, 0, 0, 0)=True) DO BEGIN     { Wait for message }
     TranslateMessage(Msg);                           { Translate message }
     DispatchMessage(Msg);                            { Dispatch message }
   END;

   UnRegisterClass(szClassName, HInstance);           { Unregister class }
   UnRegisterClass(szHoverClass, HInstance);          { Unregister class }
END.




